-- Misc extras for the Functional Images paper
-- See http://research.microsoft.com/~conal/papers/functional-images


module FIPaper where

import Int (fromInt)
import Monad
import Pan hiding (sizeUI,ybRings,bilerpBRBW)

-- import DemoUtils (ToColor,toImageC,vstrip,wavDist, compileImageFor,compileImageBFor,vsize)

import Wavies ()
import Simple (swirlUI,altRings)
import MiscDemos (gasket)
import Xor (xorRs)

backend = Component --PhotoShopFilter


-- Stretch or squeeze so that the view window shows sizeXsize units.
resize :: FloatE -> Filter c
resize size = uscale (fromInt vsize'/size)
 where
   vsize' | backend == PhotoShopFilter = 100
          | otherwise                  = viewSize

compIm :: ToColor c => String -> Image c -> FloatE -> IO ()
compIm name im size =
  compileImageFor [backend] ("fip_" ++ name) $
  return (resize size im)

{-
compIB :: ToColor c => String -> ImageB c -> FloatE -> IO ()
compIB name imb size =
  compileImageBFor backends ("fip_" ++ name) $
  return $ scaleT 5 (resize size . imb)
-}

-- Instead, turn into a still image
compIB :: ToColor c => String -> ImageB c -> TimeE -> TimeE
       -> IVectorE -> FloatE -> IO ()
compIB name imB t0 t1 isize@(cols,rows) size = 
  compIm name 
   (snapshots white isize (size,size) t0 t1 imB')
   (i2f cols * size)
 where
--    imB' t = uscale size (frame (1-borderFrac)) `over`
--             uscale (1-borderFrac) (toImageC (imB t))
   imB' t = uscale size (frame bfrac) `over`
            uscale bfrac (toImageC (imB t))
   bfrac = 1 - borderFrac * i2f cols

frame inner = bwIm reg
 where reg = (universeR {-box-} `diffR` uscale inner box)
       box (x,y) = inU x &&* inU y
       inU w = abs w <* 1/2

-- Fraction of space devoted to a border
borderFrac = 1/100 :: FloatE

----------------------------------------

type IVectorE = (IntE,IntE)

imArr :: Syntactic c => c -> IVectorE -> VectorE
      -> ((IntE,IntE) -> Image c) -> Image c
imArr def isize elemSize imF p =
  ifE outside
      def
      (translate (ecx,ecy) (imF (i,j)) p)
 where
  (cols,rows) = isize
  (w,h)       = elemSize
  (x,y)       = p

  (ecx,i )  = centerElem cols w x
  (ecy,j')  = centerElem rows h y
  j         = rows - j' - 1  -- we want top-to-bottom

  outside = i <* 0 ||* i >=* cols ||* j <* 0 ||* j >=* rows

-- 1-D version: takes number of elements, size and sample point, and
-- yields center position and element index.  Accounts for centering.
centerElem :: IntE -> FloatE -> FloatE -> (FloatE,IntE)
centerElem elems size z = (center,ix)
 where
   allSize = size * i2f elems
   ix      = floorE ((z + allSize/2) / size)
   center  = (i2f ix + 1/2) * size - allSize/2

{-
[(x,centerElem 5 2 (fromDouble x)) | x <- [-7 .. 5]]
-}

-- rows x cols snapshots at times t0, ..., t1 an animation.
snapshots :: Syntactic c => c -> IVectorE -> VectorE -> TimeE -> TimeE
          -> ImageB c -> Image c
snapshots def isize@(cols,rows) elemSize t0 t1 anim =
  imArr def isize elemSize (anim . timeF)
 where
   eps = (t1 - t0) / i2f (rows*cols - 1)
   timeF :: IVectorE -> TimeE
   timeF (c,r) = t0 + eps * i2f (r*cols+c)

snapshotsUI :: Syntactic c => c -> UI (ImageB c) -> UI (Image c)
snapshotsUI def imBUI =
  liftM5 (snapshots def)
   isizeUI sizeUI
   (slider "start time" (-50,50) 0)
   (slider "dt" (-1,10) 0.1)
   imBUI

sizeUI = liftM2 (,) (f "width") (f "height")
 where
   f str = slider str (0,200) 100

isizeUI = liftM2 (,) (f "columns") (f "rows")
 where
   f str = islider str (0,20) 4



----------------------------------------

ybRings = lerpI wavDist blueI yellowI

polarChecker n = altRings `xorR` radReg n

-- Alternative formulation of checker, alluded to in the Region Algebra
-- section.
checker' = complementR $ evenX `xorR` (evenX . swap)
 where
   evenX (x,y) = evenE (floorE x)
   swap  (x,y) = (y,x)

wedgeAnnulus :: FloatE -> IntE -> Region
wedgeAnnulus inner n = annulus inner `intersectR` radReg n

cropRad rad = crop (disk rad)
cropu       = cropRad 1

xorgon :: Int -> FloatE -> Region -> Region
xorgon n r reg = xorRs (map rf [0 .. n-1])
 where
  rf i = translate (fromPolar (r,a)) reg
   where a = fromInt i * 2*pi / fromInt n

bilerpBRBW = bilerpC black red blue white

tiledBilerp = about (1/2,1/2) (tile (1,1)) bilerpBRBW

wiggleRotateP :: FloatE -> FloatE -> TimeE -> TransformE
wiggleRotateP cycles maxAng t p = polarXf xf p
 where
   xf (r,a) = (r, a + maxAng * sin (t + dt))
    where
      dt = 2 * pi * cycles * (r - 1/2)

wiggleRotate :: FloatE -> FloatE -> TimeE -> Filter c
wiggleRotate cycles maxAng t im =
  im . wiggleRotateP cycles maxAng t

washer :: FloatE -> FloatE -> FloatE -> FilterBC
washer cycles maxAng stretch im t =
  cropRad 1 $
  wiggleRotate cycles maxAng t $
  uscale stretch $
  im

circleLimit :: FloatE -> FilterC
circleLimit radius im = cropRad radius (im . polarXf xf)
 where
  xf (r,a) = (radius * r / (radius - r), a)

cwaving' :: (FloatE -> FloatE) -> FloatE -> TransformE
cwaving' f period = polarXf xf
 where
  xf (r,a) =(f (r * (2 * pi / period)), a)

cwave' :: (FloatE -> FloatE) -> FloatE -> Filter c
cwave' f r     im = im . cwaving'   f r

cwaveSinCropped =
  uscale 21 udisk `intersectR`
  rotate (pi/4) (cwave sin 14 checker)

impIm wid name = uscale (100/wid) $
                 importImage name

impImWH :: Int -> Int -> String -> (ImageC,FloatE)
impImWH wI hI name = (im,h)
 where
   im = impIm wF name
   h  = 100 * hF/wF
   hF = fromInt hI
   wF = fromInt wI
   

becky  = impIm 204 "Becky.jpg"
fraidy = impIm 143 "fraidy-cat.bmp"

(kids  ,kidsH)   = impImWH 384 449 "kids.jpg"
(flower,flowerH) = impImWH 108 112 "flower.bmp"
(piggy ,piggyH)  = impImWH 230 227 "piggy.jpg"

-- becky   = impIm 204 "Becky.jpg"
-- fraidy  = impIm 143 "fraidy-cat.bmp"
-- kids    = impIm 384 "kids.jpg"
-- kidsH   = 100*449/384 :: FloatE
-- flower  = impIm 108 "flower.bmp"
-- flowerH = 100*112/108 :: FloatE
-- piggy   = impIm 230 "piggy.jpg"
-- piggyH  = 100*227/230

hybrid :: (TimeE -> Region) -> (TimeE -> ImageC)
hybrid f t = cond (f t) fraidy becky

turningXPos  t = rotate t xPos
swirlingXPos t = swirl (10 / sin t) xPos
roamingDisk  t = uscale 30 (translate (cos t, sin (2*t)) udisk)

wipe :: FloatE -> FractionE
wipe x = ifE (x <* -0.5) 0 (ifE (x <* 0.5) (0.5 + x) 1)

wipe2   :: FloatE -> Image FractionE
wipe2 w = uscale w (wipe . fst)

swirlWipe = swirl 20 (wipe2 50)
rotateWipe  = rotate (-pi/4) (wipe2 50)

bwIm, byIm :: Region -> ImageC
bwIm reg = cond reg blackI whiteI
byIm reg = cond reg blueI  redI 
-- should be yellowI for paper, but looks red looks better printed

{-
doubleCircleLimitFlower =
  circleLimit 200 $ tile (420,420) $
  circleLimit 200 $ tile (105,109) $
  flower

doubleCircleLimitFlower =
  tile (420,420) $ circleLimit 200 $
  tile (105,109) $ flower

tileLimit r n im =
  (iterate cLim im) !! n
 where
   cLim = tile (2.1*r,2.1*r) . circleLimit r

tileLimit r n im =
  (iterate (tile (2.1*r,2.1*r) . circleLimit r) im) !! n
-}

tileLimit :: FloatE -> Int -> FilterC

tileLimit r n = fpow (tile (2.1*r,2.1*r) . circleLimit r) n

fpow :: (a -> a) -> Int -> (a -> a)
fpow f n a = iterate f a !! n

doubleCircleLimitPiggy  = tileLimit 200 2 (tile (105,105) piggy)

doubleCircleLimitChecker = tileLimit 10 2 (bwIm checker)

----------------------------------------

makeAll = do
  compIm "vstrip" vstrip 5
  compIm "checker" checker 7
  compIm "altRings" altRings 10
  compIm "polarChecker_10" (polarChecker 10) 10
  compIm "gasket" gasket 1440
  compIm "wavDist" wavDist 10
  compIm "bilerp" (translate (-0.5,-0.5) bilerpBRBW) 1
  compIm "lerp_checkers_a" (lerpI wavDist (bwIm (polarChecker 10)) (byIm checker)) 7
  compIm "lerp_checkers_b" (lerpI wavDist (byIm (polarChecker 10)) (bwIm checker)) 7
  compIm "lerp_checkers_c" (lerpI wavDist (byIm checker) (bwIm (polarChecker 10))) 7
  compIm "lerp_checkers_d" (lerpI wavDist (bwIm checker) (byIm (polarChecker 10))) 7
  compIm "ybRings" ybRings 10
  compIm "udisk" udisk 3
  compIm "udisk_scaled_2" (udisk . uscaleP 2) 3
  compIm "swirl_1_vstrip" (swirl 1 vstrip) 5
  compIB "swirl_3sin_vstrip" (\ t -> swirl (3*sin t) vstrip) 0.00001 pi (4,4) 5
  compIB "swirl_xPos" (\t -> swirl (t^3) xPos) 0 2 (4,4) 5
  compIm "annulus_0p5" (annulus 0.5) 3
  compIB "radReg_anim" (radReg . floorE) 0 8 (3,3) 1
  compIm "wedgeAnnulus_0p25_7" (wedgeAnnulus 0.25 7) 2.5
  compIm "ringon_2" (xorgon 2 2.6 altRings) 10
  compIm "ringon_8" (xorgon 8 (7/4) altRings) 7
  compIm "tiled_kids" (tile (102,120) kids) 450
  compIm "tiledBilerp" tiledBilerp 3
  compIm "wedgeAnnulus_crop_ybRings" (crop (wedgeAnnulus 0.25 10) ybRings) 2.5
  compIm "wedgeAnnulus_swirl_crop_ybRings" (crop (swirl 2 (wedgeAnnulus 0.25 10)) ybRings) 2.5
  compIm "radInvert_checker" (radInvert checker) 2.2
  compIm "rippleRadius_8_0p3_ybRings" (rippleRadius 8 0.3 ybRings) 10
  compIB "rippleRadius_anim" (\ t -> rippleRadius 8 (cos t / 2) ybRings) 0 pi (3,3) 10
  compIm "ripple_crop" (rippleRadius 8 0.3 $ cropRad 5 $ ybRings) 15
  compIm "crop_ripple" (cropRad 5 $ rippleRadius 8 0.3 $ ybRings) 15
  compIm "swirl_crop_ripple" (swirl 8 $ cropRad 5 $ rippleRadius 5 0.3 $ ybRings) 15
  compIm "swirl_ripple_crop" (swirl 8 $ rippleRadius 5 0.3 $ cropRad 5 $ ybRings) 15
  compIm "cwave_sin_cropped" cwaveSinCropped 45
  compIm "cwave_tan" (cwave tan 10 checker) 22
  compIB "washer" (washer (1/2) (pi/2) 1 tiledBilerp) 0 pi (4,2) 2.1
  compIm "circleLimit_checker" (circleLimit 10 (bwIm checker)) 22
  compIm "circleLimit_kids" (circleLimit 200 (tile (102,120) kids)) 450
  compIm "double_circleLimit_checker" doubleCircleLimitChecker 22
  compIm "double_circleLimit_piggy" doubleCircleLimitPiggy 450
  compIB "hybrid_turningXPos"  (hybrid turningXPos ) 0 pi (4,2) 120
  compIB "hybrid_swirlingXPos" (hybrid swirlingXPos) 0 pi (4,2) 100
  compIB "hybrid_roamingDisk"  (hybrid roamingDisk ) 0 pi (4,2) 100
  compIm "soft_hybrid_swirl" (lerpI (swirl 10 (wipe2 75)) becky fraidy) 100
