-- Misc demos extracted from the old Demo.hs

module MiscDemos where

import Monad
import Pan
import Simple (rippleUI,tileUI',spinUI)

ybRingsUI = do mag <- slider "scale rings" (0,10) 1
               return $
                 uscale mag $ ybRings

cropRingsUI = liftM (crop (disk 50)) ybRingsUI

rippleCropRingsUI = ap rippleUI cropRingsUI


-- Sierpinski's gasket (roughly)
gasket :: Region
gasket p = i .|. j <=* i where (i,j) = f2iPair p

-- No controls on this one
gasketUI :: UI ImageC
gasketUI = return $ cond gasket blackI whiteI

-- Sinusoidal horizontal and vertical displacement.  ToDo: better names.
wibbleUI, woobleUI :: UI (Filter c)
[wibbleUI,woobleUI] = map g [swap, id]
 where
   g h = liftM3 f (slider  "scale" (0,200) 10)
                  fullAngleUI
                  (slider  "magnitude" (0,1) 0.5)
    where
      f sc ang mag = atUScale sc $ atRotate ang (. xf mag)
      xf mag = \ (x,y) -> let (u,v) = h (x,y) in
                            (x + mag * cos u, y + mag * sin v)
   swap (x,y) = (y,x)

-- ToDo: use fromPolar (h (x,y)) in previous example

wibbleTileUI = wibbleUI `ap` tileUI'
woobleTileUI = woobleUI  `ap` tileUI'

-- wibble1 period mag is a monotonic function f such that f(x)-x is
-- sinusoidal with given period and magnitude.
wibble1 :: FloatE -> FractionE -> FractionE -> Transform1E
wibble1 period phase mag x = x + mag * sin (2*pi*phase + x*u) / u
 where
   u = 2*pi/period

wibble1UI :: UI Transform1E
wibble1UI = liftM3 wibble1 (slider "period"    (0,200) 30)
                           (slider "phase"     (0,1  ) 0)
                           (slider "magnitude" (-1, 1) 0.5)

-- Doesn't look good.  I think it needs to have the lightness tweaked as well.
pondUI :: UI (Filter c)
pondUI = liftM (\ xf im -> im . radialXf xf) wibble1UI

pondUI' :: UI (Filter c)
pondUI' = liftM3 h (islider "periods" (1,14) 2)
                   (slider  "phase"   (0,1 ) 0)
                   (slider  "magnitude" (-1,1) 0.5)
 where
   h periods phase mag = (. angularXf (wibble1 per phase mag))
     where
       per = 2*pi / i2f periods

-- pondUI' = liftM (\ xf im -> im . angularXf xf) wibble1UI


-- Inner/outer swirls
iswirlUI :: UI (Filter a)
iswirlUI = liftM3 iswirl
             (tick "inner" True)
             (slider "radius" (0,50) 25)
             (slider "max rotations" (-1,1) 0.25)

circlify' :: SizedImage a -> SizedImage a
circlify' (w,h,im) = (resW,resW,im . f . toPolar)
 where rad     = w/(2*pi)
       f (r,a) = (-a*rad, r-rad)
       resW    = 2*rad + h

circlify :: SizedImageC -> ImageC
circlify szIm = im
 where (_,_,im) = circlify' szIm

{-
circlify :: SizedImageC -> ImageC
circlify (w,h,im) = im . f . toPolar
 where rad     = w/(2*pi)
       f (r,a) = (-a*rad, r-rad)
-}

circlifyUI :: UI ImageBC
circlifyUI = spinUI `ap`
             liftM circlify (imageSourceSize "source")


-- Round lens.  From a Java applet (much slower and less readable).  See
-- http://dev-gamelan.earthweb.com/journal/techworkshop/010500_ve1.html
-- My thanks to Mike Stout for pointing me to this example.

lensP :: BoolE -> FloatE -> FloatE -> TransformE
lensP alsoOutside maxMag radius =  radialScaleXf $ \ r -> 
  ifE (r <* radius ||* alsoOutside)
      (recip (maxMag - (maxMag-1) * r / radius))
      1

lens :: BoolE -> FloatE -> FloatE -> Filter c
lens onlyInLens maxMag radius = (. lensP onlyInLens maxMag radius)

lensUI :: UI (Filter c)
lensUI = liftM3 lens
           (tick "also outside lens" False)
           (slider "magnification" (-8,8) 2)
           (slider "radius" (0,200) 40)

lensPowerUI :: UI (Filter c)
lensPowerUI = liftM3 lens
                (tick "also outside lens" False)
                (liftM (2**) (slider "power magnification" (-10,5) 1))
                (slider "radius" (0,200) 40)

----------------------------------------

makeAll = makeAllFor allBackends

makeAllFor bends = do
  compileImageFor  bends "ripple rings"  rippleCropRingsUI
  compileImageFor  bends "gasket"        gasketUI
  compileFilterFor bends "wibble"        $ atMouse wibbleUI
  compileFilterFor bends "wooble"        $ atMouse woobleUI
  compileFilterFor bends "iswirl"        $ atMouse iswirlUI
  compileImageFor  bends "wibble tiled"  wibbleTileUI
  compileImageFor  bends "wooble tiled"  woobleTileUI
  compileImageBFor bends "circlify"      circlifyUI
  compileFilterFor bends "lens"          $ atMouse lensUI
  compileImageFor  bends "lens checker"  $ liftM ($ cb0) (atMouse lensUI)
  compileImageFor  bends "lens power checker"  $ liftM ($ cb0) (atMouse lensPowerUI)
  compileImageFor  bends "lens tiled"    $ atMouse lensUI `ap` tileUI'
