-- More Op Art examples

module OpArt where

import Monad
import Pan
import Simple (tileUI,powerScaleUI,spinUI, cropUI,swirlUI,tileUI')
import Wavies (cropBulgeUI)
import Inversion (invertUI)

cover :: FractionE -> FloatE -> BoolE
cover coverage x = fracPart (x+coverage/2) <* coverage

coverageUI :: UI FloatE
coverageUI = slider "coverage" (0,1) 0.5

diags :: FractionE -> Region
diags coverage (x,y) = cover coverage (x+y)

diagsUI = liftM (uscale 20 . diags) coverageUI

frames :: Region
frames (x,y) = evenE (roundE (abs x `max` abs y))

framesUI :: UI Region
framesUI = return (uscale 20 frames)


-- From Reginald Neal's "Square of Three", 1964
-- http://web.inter.nl.net/hcc/Hans.Kuiper/optica12.htm

neal :: FloatE -> IntE -> Region
neal coverage frameW = condFilter reg id hflip (diags coverage)
 where
   reg = uscale (i2f frameW / 2) frames

nealUI :: UI Region
nealUI = liftM (uscale 20) $
         liftM2 neal
           coverageUI
           (islider "frame width" (0,20) 4)


nealTiled :: FloatE -> IntE -> IntE -> IntE -> Region
nealTiled coverage frameW tileX tileY =
  tile (i2f (frameW+tileX), i2f (frameW+tileY)) $
  neal coverage frameW


nealTiledUI :: UI Region
nealTiledUI = liftM (uscale 10) $
              liftM4 nealTiled
                coverageUI
                (islider "frame width" (0,20) 4)
                (islider "tile X" (1,20) 4)
                (islider "tile Y" (1,20) 4)


nealTiledBulgedUI = cropBulgeUI  `ap` liftM toImageC nealTiledUI
nealTiledInvertUI = invertUI     `ap` nealTiledUI


-- Apply one of two filters.  Generally useful.  Move to Image.hs
condFilter :: Syntactic a => Region -> Filter a -> Filter a -> Filter a
condFilter reg pos neg im = cond reg (pos im) (neg im)


----------------------------------------

makeAll = makeAllFor
            -- [Component,PhotoShopFilter]
            allBackends

makeAllFor bends = do
  compileImageFor bends "diags"  $ diagsUI
  compileImageFor bends "frames" $ framesUI
  compileImageFor bends "neal"   $ nealUI
  compileImageFor bends "neal tiled"   $ nealTiledUI
  compileImageFor bends "neal tiled bulged"   $ nealTiledBulgedUI
  compileImageFor bends "neal tiled invert"   $ nealTiledInvertUI
