// PerfData.cpp: implementation of the CPerfData class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "PerfData.h"
#include <stdio.h>


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CPerfData::CPerfData(CStatusBar* bar)
{
    m_bar = bar;
    m_fps = 0;
    m_startMS = timeGetTime();
    m_ticks = 0;
    m_enableFPS = false;
}

CPerfData::~CPerfData()
{

}

//
// FUNCTION: void EnableFPS(bool)
//
// PURPOSE:  Toggle the reporting of FPS perf data.
//
void CPerfData::EnableFPS(bool f)
{
    m_ticks   = 0;
    m_startMS = timeGetTime();
    m_enableFPS = f;
}

//
// FUNCTION: void SetPixelsPerSec(float)
//
// PURPOSE:  Report the pixels/sec count for the most
//           recent frame.
// 
void CPerfData::SetPixelsPerSec(float pps, int pass)
{
    TCHAR msg[255];

    _stprintf(msg, TEXT("%.0f pixels/sec"), pps);
    m_bar->SetColumnText(1,msg);

    if (m_enableFPS) {
        m_ticks++;
        m_fps = (float)((float)m_ticks / ((timeGetTime() - m_startMS)/1000.0));

        _stprintf(msg, TEXT("%.2f frames/sec"), m_fps);
        m_bar->SetColumnText(2,msg);
    } else {
        _stprintf(msg, TEXT("pass # %d"), pass);
        m_bar->SetColumnText(2,msg);
    }
}

void CPerfData::SetViewSize(int w,int h)
{
    TCHAR msg[255];

    _stprintf(msg,TEXT("%u x %u pixels"), w,h);
    m_bar->SetColumnText(0,msg);
    // Changing the view size should reset the fps info:
    if (m_enableFPS) EnableFPS(true);
}
